/*
 * Copyright (c) 2009, Takeyuki Nagao
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the
 * following conditions are met:
 * 
 *  * Redistributions of source code must retain the above
 *    copyright notice, this list of conditions and the
 *    following disclaimer.
 *  * Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *    
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 */

package jp.sourceforge.dvibrowser.dvicore.util.concurrent;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.ThreadFactory;

import jp.sourceforge.dvibrowser.dvicore.util.DaemonThreadFactory;


public class ThreadedComputer<K, V> extends AbstractComputer<K, V> {
//  private static final Logger LOGGER = Logger.getLogger(BasicComputer.class
//      .getName());
  private final ExecutorService executorService;
  
  public ThreadedComputer(int nThreads, ThreadFactory threadFactory)
  {
    if (threadFactory == null) {
      threadFactory = createThreadFactory();
    }
    executorService = createExecutorService(nThreads, threadFactory);
  }

  public ThreadedComputer(int nThreads)
  {
    this(nThreads, null);
  }
  
  protected ThreadFactory createThreadFactory()
  {
    return new DaemonThreadFactory(Thread.MIN_PRIORITY);
  }
  
  protected ExecutorService createExecutorService(int nThreads, ThreadFactory threadFactory)
  {
    return Executors.newFixedThreadPool(nThreads, threadFactory);
  }

  public ExecutorService getExecutorService()
  {
    return executorService;
  }

  @Override
  public ExecutorService getExecutorService(Computation<K, V> computable)
  {
    return executorService;
  }
}
